# Adds two integers together.
# @param x The first integer to add.
# @param y The second integer to add.
# @return [Integer] Returns the sum of x and y.
# @example Example of adding two integers.
#   test::add(1, 2) => 3
function test::add(Integer $x, Integer $y) {
  $x + $y
}
