# The module for custom YARD code objects.
module PuppetStrings::Yard::CodeObjects
  require 'puppet-strings/yard/code_objects/class'
  require 'puppet-strings/yard/code_objects/defined_type'
  require 'puppet-strings/yard/code_objects/type'
  require 'puppet-strings/yard/code_objects/provider'
  require 'puppet-strings/yard/code_objects/function'
  require 'puppet-strings/yard/code_objects/task'
  require 'puppet-strings/yard/code_objects/plan'
end
