<% even = false  %>
<% @items.each do |item| %>
    <li id="object_<%=item.path%>" class="<%= even ? 'even' : 'odd' %>">
      <div class="item">
        <%= linkify item, h(item.name(true)) %>
        <small>Resource type: <em><%=item.type_name%></em></small>
      </div>
    </li>
    <% even = !even %>
<% end %>
