# The module for custom YARD tags.
module PuppetStrings::Yard::Tags
  require 'puppet-strings/yard/tags/parameter_directive'
  require 'puppet-strings/yard/tags/property_directive'
  require 'puppet-strings/yard/tags/overload_tag'
  require 'puppet-strings/yard/tags/summary_tag'
end
